@@ -1,11 +1,14 @@
-from typing import Generic, TypeVar, Callable, overload, Optional, Union
+from typing import *  # type: ignore
 from typing_extensions import Self
 
 S = TypeVar("S")
 T = TypeVar("T")
 
 
-def eval_or_discard(mapping, namespace=None):
+def eval_or_discard(
+    mapping: Mapping[str, T],
+    namespace: Optional[Dict[str, Any]] = None,
+) -> Mapping[object, T]:
     result = {}
 
     for key, value in mapping.items():
@@ -29,14 +32,14 @@ class cached_property(Generic[S, T]):
         self._getter = getter
 
     @overload
-    def __get__(self, instance: S, owner: None = None) -> T:
+    def __get__(self, instance: S, owner: Optional[type] = None) -> T:
         ...
 
     @overload
-    def __get__(self, instance: None, owner: None = None) -> Self:
+    def __get__(self, instance: None, owner: Optional[type] = None) -> Self:
         ...
 
-    def __get__(self, instance: Optional[S], owner: object = None) -> Union[T, Self]:
+    def __get__(self, instance: Optional[S], owner: Optional[type] = None) -> Union[T, Self]:
         if instance is None:
             return self
 
