@@ -1,11 +1,10 @@
-
 import traceback
-import typing
 import warnings
+from typing import *  # type: ignore
 
 __all__ = [
-    'print_exception',
-    'warn_from_exception',
+    "print_exception",
+    "warn_from_exception",
 ]
 
 
@@ -13,7 +12,7 @@ def print_exception(exception: BaseException) -> None:
     """
     Shorthand for ``traceback.print_exception(type(exc), exc,
     exc.__traceback__)``.
-    
+
     :param exception: The exception to print
     """
     traceback.print_exception(type(exception), exception, exception.__traceback__)
@@ -21,8 +20,8 @@ def print_exception(exception: BaseException) -> None:
 
 def warn_from_exception(
     exception: BaseException,
-    warning_type: typing.Type[Warning] = UserWarning,
-    message: typing.Optional[str] = None,
+    warning_type: Type[Warning] = UserWarning,
+    message: Optional[str] = None,
 ) -> None:
     """
     Turns the given exception into a warning, forwarding all the relevant
@@ -33,19 +32,25 @@ def warn_from_exception(
     """
     if message is None:
         message = str(exception)
-    
-    tb = exception.__traceback__
-    frame = tb.tb_frame
-    code = frame.f_code
 
-    module_globals = frame.f_globals
+    tb = exception.__traceback__
+    if tb is None:
+        filename = "<unknown file>"
+        lineno = 0
+        module_globals = {}
+    else:
+        frame = tb.tb_frame
+
+        filename = frame.f_code.co_filename
+        lineno = tb.tb_lineno
+        module_globals = frame.f_globals
 
     warnings.warn_explicit(
         message,
         warning_type,
-        filename=code.co_filename,
-        lineno=tb.tb_lineno,
-        module=module_globals.get('__name__'),
-        registry=module_globals.get('__warningregistry__'),
+        filename=filename,
+        lineno=lineno,
+        module=module_globals.get("__name__"),
+        registry=module_globals.get("__warningregistry__"),
         module_globals=module_globals,
     )
