@@ -1,5 +1,6 @@
+import ast
 import builtins
-import collections
+import collections.abc
 import importlib
 import types
 import typing
@@ -17,16 +18,13 @@ from .introspection import (
 )
 from .i_hate_circular_imports import parameterize
 from ..parameter import Parameter
-from ..signature import Signature
-from ..types import Type_, TypeAnnotation
+from ..signature_ import Signature
+from ..types import Type_, TypeAnnotation, ForwardRefContext
 from ..errors import *
 
 __all__ = ["resolve_forward_refs", "annotation_to_string", "annotation_for_callable"]
 
 
-ForwardRefContext = typing.Union[None, type, types.FunctionType, types.ModuleType, str]
-
-
 @overload
 def resolve_forward_refs(
     annotation: TypeAnnotation,
@@ -42,20 +40,22 @@ def resolve_forward_refs(
     annotation: TypeAnnotation,
     context: ForwardRefContext = None,
     *,
-    mode: Literal["eval", "getattr", "mixed"] = "mixed",
+    mode: Literal["eval", "getattr", "ast"] = "eval",
     strict: bool = True,
+    extra_globals: typing.Mapping[str, object] = {},
 ) -> TypeAnnotation:
     ...
 
 
 def resolve_forward_refs(
     annotation: TypeAnnotation,
-    module: typing.Optional[types.ModuleType] = None,
-    eval_: bool = True,
+    context: ForwardRefContext = None,
+    eval_: Optional[bool] = None,
     strict: bool = True,
     *,
-    context: ForwardRefContext = None,
-    mode: Literal["eval", "getattr", "mixed"] = "mixed",
+    module: typing.Optional[types.ModuleType] = None,
+    mode: Literal["eval", "getattr", "ast"] = "eval",
+    extra_globals: typing.Mapping[str, object] = {},
 ) -> TypeAnnotation:
     """
     Resolves forward references in a type annotation.
@@ -67,19 +67,23 @@ def resolve_forward_refs(
         >>> resolve_forward_refs('ellipsis')
         <class 'ellipsis'>
 
+    Using `mode='ast'` makes partial evaluation possible::
+
+        >>> resolve_forward_refs('List[ThisCantBeResolved'], mode='ast', strict=False)
+        List['ThisCantBeResolved']
+
     .. versionchanged:: 1.6
         The ``module`` and ``eval_`` parameters are deprecated in favor of
-        ``context`` and ``mode``. Once the ``eval_`` parameter is removed,
-        the default ``mode`` will be changed to ``'mixed'``.
+        ``context`` and ``mode``.
 
     :param annotation: The annotation in which forward references should be resolved
     :param context: The context in which forward references will be evaluated.
         Can be a class, a function, a module, or a string representing a module
         name. If ``None``, a namespace containing some common modules like
         ``typing`` and ``collections`` will be used.
-    :param mode: If ``True``, references may contain arbitrary code that will
-        be evaluated with ``eval``. Otherwise, they must be identifiers and will
-        be resolved with ``getattr``.
+    :param mode: If ``'eval'``, references may contain arbitrary code that will
+        be evaluated with ``eval``. If ``'getattr'``, they must be identifiers
+        and will be resolved with ``getattr``.
     :param strict: Whether to raise an exception if a forward reference can't be resolved
     :return: A new annotation with no forward references
     """
@@ -91,7 +95,7 @@ def resolve_forward_refs(
         )
 
     if eval_ is not None:
-        mode = "eval" if eval_ else "getattr"
+        mode = "eval" if eval_ else "ast"
         warnings.warn(
             'The "eval_" parameter is deprecated in favor of "mode"',
             DeprecationWarning,
@@ -101,37 +105,45 @@ def resolve_forward_refs(
         context = importlib.import_module(context)
 
     if isinstance(annotation, ForwardRef):
+        if context is None:
+            context = annotation.__forward_module__
+
         annotation = _get_forward_ref_code(annotation)
 
     if isinstance(annotation, str):
+        scope = collections.ChainMap()
+        scope.maps.append(extra_globals)  # type: ignore
+
+        if context is None:
+            scope.maps.extend(vars(module) for module in (builtins, typing, collections.abc, collections))  # type: ignore
+        elif isinstance(context, types.ModuleType):
+            scope.maps.append(vars(context))
+        elif isinstance(context, str):
+            module = importlib.import_module(context)
+            scope.maps.append(vars(module))
+        elif isinstance(context, collections.abc.Mapping):
+            scope.maps.append(context)  # type: ignore
+        else:
+            module = importlib.import_module(context.__module__)
+            scope.maps.append(vars(module))
+
         if mode == "eval":
-            # Eval the code in a ChainMap so that names
-            # can be resolved in the given module or in
-            # the typing module
-            modules: List[types.ModuleType] = [typing]
-            if isinstance(context, types.ModuleType):
-                modules.insert(0, context)
-
-            scope = collections.ChainMap(*[vars(mod) for mod in modules])
-
             try:
-                # the globals must be a real dict, so the scope will be
-                # used as the locals
+                # The globals must be a real dict, so the scope will be used as
+                # the locals
                 annotation = eval(annotation, {}, scope)
             except Exception:
                 pass
             else:
-                return resolve_forward_refs(annotation, context, eval_, strict)
+                return resolve_forward_refs(annotation, context, mode=mode, strict=strict)
         elif mode == "getattr":
-            # try to resolve the annotation in various modules
-            modules = [typing, builtins]
-            if isinstance(context, types.ModuleType):
-                modules.insert(0, context)
+            first_name, *attrs = annotation.split(".")
 
-            attrs = annotation.split(".")
-
-            for mod in modules:
-                value = mod
+            try:
+                value = scope[first_name]
+            except KeyError:
+                pass
+            else:
                 try:
                     for attr in attrs:
                         value = getattr(value, attr)
@@ -139,8 +151,17 @@ def resolve_forward_refs(
                     return value  # type: ignore
                 except AttributeError:
                     pass
+        elif mode == "ast":
+            expr = ast.parse(annotation, mode="eval")
+
+            try:
+                result = _eval_ast(expr.body, scope, strict=strict)
+            except _AstEvaluationFailed:
+                pass
+            else:
+                return result  # type: ignore
         else:
-            raise NotImplementedError  # FIXME
+            assert False, f"Invalid mode: {mode!r}"
 
         if annotation == "ellipsis":
             return type(...)
@@ -148,13 +169,46 @@ def resolve_forward_refs(
         if not strict:
             return annotation
 
-        raise CannotResolveName(annotation, context)
+        raise CannotResolveForwardref(annotation, context)
 
+    if annotation is None:
+        return None
+
+    # In some versions, `ParamSpec` is a subclass of `list`, so make sure this
+    # check happens before the `isinstance(annotation, list)` check below
+    if type(annotation) in (
+        typing_extensions.TypeVarTuple,
+        typing_extensions.ParamSpec,
+        typing_extensions.ParamSpecKwargs,
+        typing_extensions.ParamSpecArgs,
+    ):
+        return annotation
+
+    if isinstance(annotation, TypeVar):
+        if annotation.__constraints__:
+            constraints = [
+                resolve_forward_refs(constraint, context, mode=mode, strict=strict)
+                for constraint in annotation.__constraints__
+            ]
+            bound = None
+        else:
+            constraints = ()
+            bound = resolve_forward_refs(annotation.__bound__, context, mode=mode, strict=strict)
+
+        return TypeVar(
+            annotation.__name__,  # type: ignore
+            *constraints,  # type: ignore
+            bound=bound,  # type: ignore
+            covariant=annotation.__covariant__,  # type: ignore
+            contravariant=annotation.__contravariant__,  # type: ignore
+        )
+
+    # As a special case, lists are also supported because they're used by
+    # `Callable`
     if isinstance(annotation, list):
         return [
-            resolve_forward_refs(typ, context, mode=mode, strict=strict)
-            for typ in annotation
-        ]
+            resolve_forward_refs(typ, context, mode=mode, strict=strict) for typ in annotation
+        ]  # type: ignore
 
     if not is_parameterized_generic(annotation, raising=False):
         return annotation
@@ -166,22 +220,72 @@ def resolve_forward_refs(
 
     type_args = get_type_arguments(annotation)
     type_args = tuple(
-        resolve_forward_refs(typ, context, mode=mode, strict=strict)
-        for typ in type_args
+        resolve_forward_refs(typ, context, mode=mode, strict=strict) for typ in type_args
     )
     return parameterize(base, type_args)
 
 
+def _eval_ast(node: ast.AST, scope: typing.Mapping[str, object], strict: bool) -> object:
+    # Compared to "eval" and "getattr", this method of evaluating forward refs
+    # has the advantage of being able to perform partial evaluation. For
+    # example, the forward ref `"ClassVar[NameThatCannotBeResolved]"` can be
+    # turned into `ClassVar["NameThatCannotBeResolved"]`.
+    #
+    # Sometimes we need to know whether the forward ref was resolved or not.
+    # That's why this function returns a tuple of `(bool, object)`.
+    def recurse(node: ast.AST) -> object:
+        return _eval_ast(node, scope, strict)
+
+    if strict:
+        safe_recurse = recurse
+    else:
+
+        def safe_recurse(node: ast.AST) -> object:
+            try:
+                return _eval_ast(node, scope, strict)
+            except Exception:
+                return ast.unparse(node)
+
+    if type(node) is ast.Name:
+        name = node.id
+        return scope[name]
+    elif type(node) is ast.Attribute:
+        obj = recurse(node.value)
+        return getattr(obj, node.attr)
+    elif type(node) is ast.Subscript:
+        generic_type = recurse(node.value)
+        subtype = safe_recurse(node.slice)
+        return generic_type[subtype]  # type: ignore
+    elif type(node) is ast.Constant:
+        return node.value
+    elif type(node) is ast.Tuple:
+        return tuple(safe_recurse(elt) for elt in node.elts)
+    elif type(node) is ast.List:
+        return [safe_recurse(elt) for elt in node.elts]
+    elif type(node) is ast.BinOp:
+        if type(node.op) is ast.BitOr:
+            left = safe_recurse(node.left)
+            right = safe_recurse(node.right)
+            # Use `Union` instead of `|` because:
+            # 1. It works in older python versions
+            # 2. It works even if `left` and `right` are strings because they
+            #    couldn't be resolved
+            return Union[left, right]  # type: ignore
+
+    raise NotImplementedError(f"Can't evaluate AST node {node}")
+
+
 def annotation_to_string(
     annotation: TypeAnnotation,
     *,
     implicit_typing: bool = True,
     new_style_unions: bool = True,
     optional_as_union: bool = True,
+    variance_prefixes: bool = False,
 ) -> str:
     """
-    Converts a type annotation to string. The result is
-    valid python code.
+    Converts a type annotation to string. The result is valid python code
+    (unless ``variance_prefixes`` is set to ``True``).
 
     Examples::
 
@@ -197,18 +301,21 @@ def annotation_to_string(
         types' names
     :param new_style_unions: Whether to use the new-style ``typing.Union`` syntax
         ``int | str`` instead of ``Union[int, str]``
+    :param variance_prefixes: Whether `TypeVars` and `ParamSpecs` should be
+        prefixed with ``+``, ``-``, or ``~`` to indicate variance
     :return: A string that, when evaluated, returns ``annotation``
     """
 
-    def recurse(annotation):
+    def recurse(annotation: TypeAnnotation) -> str:
         return annotation_to_string(
             annotation,
             implicit_typing=implicit_typing,
             new_style_unions=new_style_unions,
             optional_as_union=optional_as_union,
+            variance_prefixes=variance_prefixes,
         )
 
-    def process_nested(prefix, elems):
+    def process_nested(prefix: str, elems: Iterable[TypeAnnotation]):
         elems = ", ".join(recurse(ann) for ann in elems)
         return "{}[{}]".format(prefix, elems)
 
@@ -239,7 +346,26 @@ def annotation_to_string(
         return process_nested(prefix, subtypes)
 
     if isinstance(annotation, (typing.TypeVar, typing_extensions.ParamSpec)):
-        return str(annotation)
+        result = annotation.__name__
+
+        if variance_prefixes:
+            if annotation.__covariant__:
+                result = "+" + result
+            elif annotation.__contravariant__:
+                result = "-" + result
+            else:
+                result = "~" + result
+
+        return result
+
+    if isinstance(annotation, typing_extensions.TypeVarTuple):
+        return annotation.__name__
+
+    if isinstance(annotation, typing_extensions.ParamSpecArgs):
+        return recurse(annotation.__origin__) + ".args"
+
+    if isinstance(annotation, typing_extensions.ParamSpecKwargs):
+        return recurse(annotation.__origin__) + ".kwargs"
 
     if hasattr(annotation, "__module__"):
         if annotation.__module__ == "builtins":
@@ -257,7 +383,7 @@ def annotation_to_string(
     return repr(annotation)
 
 
-def annotation_for_callable(callable_: typing.Callable) -> Type_:
+def annotation_for_callable(callable_: typing.Callable[..., object]) -> Type_:
     """
     Given a callable object as input, returns a matching type annotation.
 
@@ -299,7 +425,5 @@ def annotation_for_callable(callable_: typing.Callable) -> Type_:
     if len(options) == 1:
         return typing.Callable[param_types, return_type]  # type: ignore
 
-    options = tuple(
-        typing.Callable[option, return_type] for option in options  # type: ignore
-    )
+    options = tuple(typing.Callable[option, return_type] for option in options)  # type: ignore
     return typing.Union[options]  # type: ignore
