@@ -4,22 +4,25 @@ import collections.abc
 import re
 import sys
 import types
-import typing
 import typing_extensions
+from typing import *  # type: ignore
 
 from introspection.typing import *
 from introspection import errors
+from introspection.types import Type_
+
+from typing import Type  # overwrite the `Type` imported from `introspection.typing`
 
 
 is_py39_plus = sys.version_info >= (3, 9)
 
 
-T = typing.TypeVar("T")
-T_co = typing.TypeVar("T_co", covariant=True)
-E = typing.TypeVar("E", bound=Exception)
+T = TypeVar("T")
+T_co = TypeVar("T_co", covariant=True)
+E = TypeVar("E", bound=Exception)
 
 
-class MyGeneric(typing.Generic[E]):
+class MyGeneric(Generic[E]):
     pass
 
 
@@ -38,9 +41,9 @@ class UnhashableClass(metaclass=UnhashableMeta):
         (list, False),
         (None, False),
         (UnhashableClass, False),
-        (typing.List, False),
+        (List, False),
         ("Foo", True),
-        (typing.ForwardRef("Foo"), True),
+        (ForwardRef("Foo"), True),
     ],
 )
 def test_is_forwardref(type_, expected):
@@ -87,27 +90,27 @@ def test_is_forwardref_non_raising(obj):
         ("Foo", True),  # this is a forward reference
         (T_co, True),
         (E, True),
-        (typing.Any, True),
-        (typing.List, True),
-        (typing.Union, True),
-        (typing.Callable, True),
-        (typing.Optional, True),
-        (typing.Type, True),
-        (typing.NoReturn, True),
+        (Any, True),
+        (List, True),
+        (Union, True),
+        (Callable, True),
+        (Optional, True),
+        (Type, True),
+        (NoReturn, True),
         (MyGeneric, True),
-        (typing.TypeVar, True),
-        (typing.List[int], True),
-        (typing.Union[int, str], True),
-        (typing.Callable[[], int], True),
-        (typing.Optional[int], True),
-        (typing.ByteString, True),
-        (typing.List[E], True),
+        (TypeVar, True),
+        (List[int], True),
+        (Union[int, str], True),
+        (Callable[[], int], True),
+        (Optional[int], True),
+        (ByteString, True),
+        (List[E], True),
         (MyGeneric[E], True),
         (MyGeneric[int], True),
-        (typing.List[typing.Tuple[E]], True),
-        (typing.List[typing.Tuple], True),
-        (typing.List[typing.Callable[[E], int]], True),
-        (typing.List[typing.Callable], True),
+        (List[Tuple[E]], True),
+        (List[Tuple], True),
+        (List[Callable[[E], int]], True),
+        (List[Callable], True),
         (typing_extensions.Protocol, True),
     ],
 )
@@ -119,7 +122,7 @@ def test_is_type(type_, expected):
     "type_, expected",
     [
         ("Foo", False),
-        (typing.ForwardRef("Foo"), False),
+        (ForwardRef("Foo"), False),
     ],
 )
 def test_is_type_no_forwardref(type_, expected):
@@ -134,30 +137,30 @@ def test_is_type_no_forwardref(type_, expected):
         (UnhashableClass, False),
         (None, False),
         ("List", False),
-        (typing.ForwardRef("List"), False),
+        (ForwardRef("List"), False),
         (T_co, True),
         (E, True),
-        (typing.Any, True),
-        (typing.List, True),
-        (typing.Union, True),
-        (typing.Callable, True),
-        (typing.Optional, True),
-        (typing.Type, True),
-        (typing.NoReturn, True),
+        (Any, True),
+        (List, True),
+        (Union, True),
+        (Callable, True),
+        (Optional, True),
+        (Type, True),
+        (NoReturn, True),
         (MyGeneric, False),
-        (typing.TypeVar, True),
-        (typing.List[int], True),
-        (typing.Union[int, str], True),
-        (typing.Callable[[], int], True),
-        (typing.Optional[int], True),
-        (typing.ByteString, True),
-        (typing.List[E], True),
+        (TypeVar, True),
+        (List[int], True),
+        (Union[int, str], True),
+        (Callable[[], int], True),
+        (Optional[int], True),
+        (ByteString, True),
+        (List[E], True),
         (MyGeneric[E], True),
         (MyGeneric[int], True),
-        (typing.List[typing.Tuple[E]], True),
-        (typing.List[typing.Tuple], True),
-        (typing.List[typing.Callable[[E], int]], True),
-        (typing.List[typing.Callable], True),
+        (List[Tuple[E]], True),
+        (List[Tuple], True),
+        (List[Callable[[E], int]], True),
+        (List[Callable], True),
     ],
 )
 def test_is_typing_type(type_, expected):
@@ -205,29 +208,29 @@ def test_is_typing_type_non_raising(type_):
         (collections.abc.Iterable, is_py39_plus),
         (collections.abc.Callable, is_py39_plus),
         (collections.abc.Sized, False),
-        (typing.Any, False),
-        (typing.List, True),
-        (typing.Union, True),
-        (typing.Tuple, True),
-        (typing.Callable, True),
-        (typing.Optional, True),
-        (typing.Type, True),
-        (typing.Generic, True),
+        (Any, False),
+        (List, True),
+        (Union, True),
+        (Tuple, True),
+        (Callable, True),
+        (Optional, True),
+        (Type, True),
+        (Generic, True),
         (MyGeneric, True),
-        (typing.List[int], False),
-        (typing.Union[int, str], False),
-        (typing.Tuple[int], False),
-        (typing.Callable[[], int], False),
-        (typing.Optional[int], False),
-        (typing.ByteString, False),
-        (typing.List[E], True),
-        (typing.Tuple[E], True),
+        (List[int], False),
+        (Union[int, str], False),
+        (Tuple[int], False),
+        (Callable[[], int], False),
+        (Optional[int], False),
+        (ByteString, False),
+        (List[E], True),
+        (Tuple[E], True),
         (MyGeneric[E], True),
         (MyGeneric[int], False),
-        (typing.List[typing.Tuple[E]], True),
-        (typing.List[typing.Tuple], False),
-        (typing.List[typing.Callable[[E], int]], True),
-        (typing.List[typing.Callable], False),
+        (List[Tuple[E]], True),
+        (List[Tuple], False),
+        (List[Callable[[E], int]], True),
+        (List[Callable], False),
         (typing_extensions.Protocol, True),
         (typing_extensions.Literal, True),
         (typing_extensions.Literal[1, 2], False),
@@ -283,28 +286,28 @@ def test_is_generic_non_raising(type_):
         (collections.abc.Callable, False),
         (None, False),
         ("List", False),
-        (typing.Any, False),
-        (typing.Union, True),
-        (typing.Tuple, True),
-        (typing.List, False),
-        (typing.Callable, False),
-        (typing.Optional, False),
-        (typing.Type, False),
+        (Any, False),
+        (Union, True),
+        (Tuple, True),
+        (List, False),
+        (Callable, False),
+        (Optional, False),
+        (Type, False),
         (MyGeneric, False),
-        (typing.List[int], False),
-        (typing.Union[int, str], False),
-        (typing.Tuple[int], False),
-        (typing.Callable[[], int], False),
-        (typing.Optional[int], False),
-        (typing.ByteString, False),
-        (typing.List[E], False),
-        (typing.Tuple[E], False),
+        (List[int], False),
+        (Union[int, str], False),
+        (Tuple[int], False),
+        (Callable[[], int], False),
+        (Optional[int], False),
+        (ByteString, False),
+        (List[E], False),
+        (Tuple[E], False),
         (MyGeneric[E], False),
         (MyGeneric[int], False),
-        (typing.List[typing.Tuple[E]], False),
-        (typing.List[typing.Tuple], False),
-        (typing.List[typing.Callable[[E], int]], False),
-        (typing.List[typing.Callable], False),
+        (List[Tuple[E]], False),
+        (List[Tuple], False),
+        (List[Callable[[E], int]], False),
+        (List[Callable], False),
         (typing_extensions.Literal, True),
         (typing_extensions.Literal[3], False),
     ],
@@ -354,19 +357,19 @@ def test_is_variadic_generic_non_raising(type_):
         (collections.abc.Iterator, is_py39_plus),
         (collections.abc.Callable, is_py39_plus),
         (collections.abc.Sized, False),
-        (typing.Any, False),
-        (typing.List, True),
-        (typing.Union, True),
-        (typing.Callable, True),
-        (typing.Optional, True),
-        (typing.Type, True),
+        (Any, False),
+        (List, True),
+        (Union, True),
+        (Callable, True),
+        (Optional, True),
+        (Type, True),
         (MyGeneric, True),
-        (typing.List[int], False),
-        (typing.Union[int, str], False),
-        (typing.Callable[[], int], False),
-        (typing.Optional[int], False),
-        (typing.ByteString, False),
-        (typing.List[E], False),
+        (List[int], False),
+        (Union[int, str], False),
+        (Callable[[], int], False),
+        (Optional[int], False),
+        (ByteString, False),
+        (List[E], False),
         (MyGeneric[E], False),
         (typing_extensions.Literal, True),
         (typing_extensions.Literal[1, 2], False),
@@ -410,22 +413,22 @@ def test_is_generic_base_class_error(type_):
         (UnhashableClass, False),
         (None, False),
         ("List", False),
-        (typing.Any, False),
-        (typing.List, False),
-        (typing.Union, False),
-        (typing.Callable, False),
-        (typing.Optional, False),
-        (typing.Type, False),
+        (Any, False),
+        (List, False),
+        (Union, False),
+        (Callable, False),
+        (Optional, False),
+        (Type, False),
         (MyGeneric, False),
-        (typing.Type[str], True),
-        (typing.List[int], True),
-        (typing.Union[int, str], True),
-        (typing.Callable[[], int], True),
-        (typing.Optional[int], True),
+        (Type[str], True),
+        (List[int], True),
+        (Union[int, str], True),
+        (Callable[[], int], True),
+        (Optional[int], True),
         (MyGeneric[str], True),
         (MyGeneric[E], True),
-        (typing.List[E], True),
-        (typing.List[typing.Tuple[E]], True),
+        (List[E], True),
+        (List[Tuple[E]], True),
         (typing_extensions.Literal, False),
         (typing_extensions.Literal[3], True),
         (typing_extensions.Protocol, False),
@@ -482,21 +485,21 @@ if is_py39_plus:
         (collections.abc.Iterable, False),
         (collections.abc.Callable, False),
         (collections.abc.ByteString, is_py39_plus),
-        (typing.Any, False),
-        (typing.List, False),
-        (typing.Union, False),
-        (typing.Callable, False),
-        (typing.Optional, False),
-        (typing.ByteString, True),
+        (Any, False),
+        (List, False),
+        (Union, False),
+        (Callable, False),
+        (Optional, False),
+        (ByteString, True),
         (MyGeneric, False),
-        (typing.Type[int], True),
-        (typing.List[int], True),
-        (typing.Union[int, str], True),
-        (typing.Callable[[], int], True),
-        (typing.Optional[int], True),
+        (Type[int], True),
+        (List[int], True),
+        (Union[int, str], True),
+        (Callable[[], int], True),
+        (Optional[int], True),
         (MyGeneric[str], True),
-        (typing.List[E], False),
-        (typing.List[typing.List[E]], False),
+        (List[E], False),
+        (List[List[E]], False),
         (typing_extensions.Literal, False),
         (typing_extensions.Literal[1, 2], True),
         (typing_extensions.Protocol, False),
@@ -570,11 +573,11 @@ if is_py39_plus:
 @pytest.mark.parametrize(
     "type_, expected",
     [
-        (typing.List[int], typing.List),
-        (typing.List[E], typing.List),
-        (typing.Union[int, str], typing.Union),
-        (typing.Callable[[], int], typing.Callable),
-        (typing.Optional[int], typing.Optional),
+        (List[int], List),
+        (List[E], List),
+        (Union[int, str], Union),
+        (Callable[[], int], Callable),
+        (Optional[int], Optional),
     ],
 )
 def test_get_generic_base_class(type_, expected):
@@ -606,12 +609,12 @@ def test_get_generic_base_class_typeerror(type_):
         UnhashableClass,
         collections.abc.Iterable,
         collections.abc.Callable,
-        typing.List,
-        typing.Tuple,
-        typing.Optional,
-        typing.Union,
-        typing.Callable,
-        typing.Type,
+        List,
+        Tuple,
+        Optional,
+        Union,
+        Callable,
+        Type,
     ],
 )
 def test_get_generic_base_class_valueerror(type_):
@@ -644,21 +647,21 @@ if is_py39_plus:
 @pytest.mark.parametrize(
     "type_, expected",
     [
-        (typing.List[int], (int,)),
-        (typing.Union[int, str], (int, str)),
-        (typing.Callable[[], int], ([], int)),
-        (typing.Callable[[str], int], ([str], int)),
-        (typing.Callable[..., int], (..., int)),
-        (typing.Optional[int], (int,)),
-        (typing.Type[str], (str,)),
-        (typing.List[E], (E,)),
-        (typing.Generator[E, int, E][str], (str, int, str)),
-        (typing.Tuple[E, int, E][str], (str, int, str)),
-        (typing.Callable[[E, int], E][str], ([str, int], str)),
-        (typing.Tuple[typing.List[E]][str], (typing.List[str],)),
+        (List[int], (int,)),
+        (Union[int, str], (int, str)),
+        (Callable[[], int], ([], int)),
+        (Callable[[str], int], ([str], int)),
+        (Callable[..., int], (..., int)),
+        (Optional[int], (int,)),
+        (Type[str], (str,)),
+        (List[E], (E,)),
+        (Generator[E, int, E][str], (str, int, str)),
+        (Tuple[E, int, E][str], (str, int, str)),
+        (Callable[[E, int], E][str], ([str, int], str)),
+        (Tuple[List[E]][str], (List[str],)),
         (
-            typing.Tuple[typing.List[typing.Type[E]]][str],
-            (typing.List[typing.Type[str]],),
+            Tuple[List[Type[E]]][str],
+            (List[Type[str]],),
         ),
     ],
 )
@@ -687,12 +690,12 @@ def test_get_type_arguments_typeerror(type_):
     [
         None,
         "List",
-        typing.List,
-        typing.Tuple,
-        typing.Optional,
-        typing.Union,
-        typing.Callable,
-        typing.Type,
+        List,
+        Tuple,
+        Optional,
+        Union,
+        Callable,
+        Type,
     ],
 )
 def test_get_type_arguments_valueerror(type_):
@@ -729,19 +732,19 @@ if is_py39_plus:
 @pytest.mark.parametrize(
     "type_, expected",
     [
-        (typing.List, "(~T,)"),
-        (typing.Union, "(+T_co,)"),
-        (typing.Callable, "(-A_contra, +R_co)"),
-        (typing.Optional, "(+T_co,)"),
-        (typing.Tuple, "(+T_co,)"),
-        (typing.Type, "(+CT_co,)"),
-        (typing.ByteString, "()"),
-        (typing.List[E], "(~E,)"),
-        (typing.List[int], "()"),
-        (typing.Generator[E, int, E], "(~E,)"),
-        (typing.Tuple[E, int, T_co], "(~E, +T_co)"),
-        (typing.Callable[[E, int], E][T_co], "(+T_co,)"),
-        (typing.Tuple[typing.List[T_co]], "(+T_co,)"),
+        (List, "(~T,)"),
+        (Union, "(+T_co,)"),
+        (Callable, "(-A_contra, +R_co)"),
+        (Optional, "(+T_co,)"),
+        (Tuple, "(+T_co,)"),
+        (Type, "(+CT_co,)"),
+        (ByteString, "()"),
+        (List[E], "(~E,)"),
+        (List[int], "()"),
+        (Generator[E, int, E], "(~E,)"),
+        (Tuple[E, int, T_co], "(~E, +T_co)"),
+        (Callable[[E, int], E][T_co], "(+T_co,)"),
+        (Tuple[List[T_co]], "(+T_co,)"),
         (MyGeneric, "(~E,)"),
         (typing_extensions.Protocol[E], "(~E,)"),
         (typing_extensions.ClassVar, "(+T_co,)"),
@@ -780,8 +783,8 @@ def test_get_type_parameters_typeerror(type_):
     [
         None,
         "List",
-        typing.Any,
-        typing.Generic,
+        Any,
+        Generic,
         typing_extensions.Protocol,
         typing_extensions.Literal,
     ],
@@ -827,17 +830,19 @@ if is_py39_plus:
         (type(...), "ellipsis"),
         (UnhashableClass, "UnhashableClass"),
         (MyGeneric, "MyGeneric"),
-        (typing.Any, "Any"),
-        (typing.List, "List"),
-        (typing.Tuple, "Tuple"),
-        (typing.Union, "Union"),
-        (typing.Optional, "Optional"),
-        (typing.Callable, "Callable"),
-        (typing.TypeVar, "TypeVar"),
-        (typing.Generic, "Generic"),
+        (Any, "Any"),
+        (List, "List"),
+        (Tuple, "Tuple"),
+        (Union, "Union"),
+        (Optional, "Optional"),
+        (Callable, "Callable"),
+        (TypeVar, "TypeVar"),
+        (Generic, "Generic"),
         (typing_extensions.Literal, "Literal"),
         (typing_extensions.Protocol, "Protocol"),
         (typing_extensions.ClassVar, "ClassVar"),
+        (typing_extensions.ParamSpec, "ParamSpec"),
+        (typing_extensions.Final, "Final"),
     ],
 )
 def test_get_type_name(type_, expected):
@@ -849,11 +854,11 @@ def test_get_type_name(type_, expected):
     [
         3,
         "List",
-        typing.List[int],
-        typing.Tuple[int, str],
-        typing.Optional[str],
-        typing.Union[int, float],
-        typing.Callable[..., None],
+        List[int],
+        Tuple[int, str],
+        Optional[str],
+        Union[int, float],
+        Callable[..., None],
     ],
 )
 def test_get_type_name_error(type_):
@@ -956,7 +961,7 @@ if sys.version_info >= (3, 10):
         ],
     )
     def test_uniontype_get_generic_base_class(type_):
-        assert get_generic_base_class(type_) is typing.Union
+        assert get_generic_base_class(type_) is Union
 
     @pytest.mark.parametrize(
         "type_, expected",
@@ -992,7 +997,7 @@ if sys.version_info >= (3, 10):
             ((str | T)[float], ()),
         ],
     )
-    def test_uniontype_get_type_parameters(type_, expected):
+    def test_uniontype_get_type_parameters(type_: Type_, expected: Tuple[TypeVar]):
         assert get_type_parameters(type_) == expected
 
     def test_uniontype_get_type_parameters_error():
