@@ -4,7 +4,7 @@ import builtins
 import io
 import sys
 import typing
-from typing import *
+from typing import *  # type: ignore
 
 from introspection.typing.misc import *
 
@@ -54,12 +54,33 @@ def test_annotation_to_string_old_style_unions(expected):
     [
         (None, "None"),
         (type(None), "None"),
+        (TypeVar("T"), "T"),
+        (TypeVar("T", covariant=True), "T"),
+        (TypeVar("T", contravariant=True), "T"),
+        (ParamSpec("P"), "P"),
+        (ParamSpec("P").args, "P.args"),  # type: ignore
+        (ParamSpec("P").kwargs, "P.kwargs"),  # type: ignore
     ],
 )
 def test_annotation_to_string(annotation, expected):
     assert annotation_to_string(annotation) == expected
 
 
+@pytest.mark.parametrize(
+    "annotation, expected",
+    [
+        (TypeVar("T"), "~T"),
+        (TypeVar("T", covariant=True), "+T"),
+        (TypeVar("T", contravariant=True), "-T"),
+        (ParamSpec("P"), "~P"),
+        (ParamSpec("P").args, "~P.args"),  # type: ignore
+        (ParamSpec("P").kwargs, "~P.kwargs"),  # type: ignore
+    ],
+)
+def test_annotation_to_string_with_variance_prefixes(annotation, expected):
+    assert annotation_to_string(annotation, variance_prefixes=True) == expected
+
+
 if hasattr(typing, "Literal"):
 
     @pytest.mark.parametrize(
@@ -74,16 +95,16 @@ if hasattr(typing, "Literal"):
         assert annotation_to_string(annotation) == expected
 
 
-@pytest.mark.parametrize(
-    "var, expected",
-    [
-        (TypeVar("T"), "~T"),
-        (TypeVar("T_co", covariant=True), "+T_co"),
-        (TypeVar("T_contra", contravariant=True), "-T_contra"),
-    ],
-)
-def test_annotation_to_string_typevars(var, expected):
-    assert annotation_to_string(var) == expected
+if hasattr(typing, "TypeVarTuple"):
+
+    @pytest.mark.parametrize(
+        "annotation, expected",
+        [
+            (TypeVarTuple("T"), "T"),
+        ],
+    )
+    def test_typevartuple_to_string(annotation, expected):
+        assert annotation_to_string(annotation) == expected
 
 
 @pytest.mark.parametrize(
@@ -133,6 +154,25 @@ def test_resolve_forward_refs(annotation, expected):
     assert resolve_forward_refs(annotation) == expected
 
 
+# TypeVars can't be compared with ==, so they need a specialized test function
+@pytest.mark.parametrize(
+    "annotation, expected",
+    [
+        (TypeVar("T", bound="int"), TypeVar("T", bound=int)),
+        (TypeVar("T", bound="int", covariant=True), TypeVar("T", bound=int, covariant=True)),
+        (TypeVar("T", "int", "str"), TypeVar("T", int, str)),
+    ],
+)
+def test_resolve_typevar_forward_refs(annotation, expected: TypeVar):
+    result: TypeVar = resolve_forward_refs(annotation)  # type: ignore
+
+    assert result.__name__ == expected.__name__
+    assert result.__bound__ == expected.__bound__
+    assert result.__constraints__ == expected.__constraints__
+    assert result.__covariant__ == expected.__covariant__
+    assert result.__contravariant__ == expected.__contravariant__
+
+
 if hasattr(typing, "Literal"):
 
     @pytest.mark.parametrize(
@@ -157,7 +197,7 @@ memoryview = bytearray
     ],
 )
 def test_resolve_forward_refs_in_module(annotation, module, expected):
-    ann = resolve_forward_refs(annotation, module=module)
+    ann = resolve_forward_refs(annotation, module)
     assert ann == expected
 
 
@@ -168,15 +208,15 @@ def test_resolve_forward_refs_in_module(annotation, module, expected):
     ],
 )
 def test_resolve_forward_refs_no_eval(annotation, module, expected):
-    ann = resolve_forward_refs(annotation, module=module, eval_=False)
+    ann = resolve_forward_refs(annotation, module, mode="getattr")
     assert ann == expected
 
 
 @pytest.mark.parametrize(
     "annotation, kwargs",
     [
-        ("ThisClassDoesntExist", {"eval_": False}),
-        ("ThisClassDoesntExist", {"module": "sys"}),
+        ("ThisClassDoesntExist", {"mode": "getattr"}),
+        ("ThisClassDoesntExist", {"context": "sys"}),
         ("this is a syntax error", {}),
     ],
 )
@@ -188,9 +228,9 @@ def test_resolve_forward_refs_error(annotation, kwargs):
 @pytest.mark.parametrize(
     "annotation, kwargs, expected",
     [
-        ("ThisClassDoesntExist", {"eval_": False}, "ThisClassDoesntExist"),
+        ("ThisClassDoesntExist", {"mode": "getattr"}, "ThisClassDoesntExist"),
         ("this is a syntax error", {}, "this is a syntax error"),
-        ('List["Foo"]', {}, List["Foo"]),
+        ('List["Foo"]', {}, List["Foo"]),  # type: ignore[Foo-not-defined]
     ],
 )
 def test_resolve_forward_refs_non_strict(annotation, kwargs, expected):
@@ -198,6 +238,17 @@ def test_resolve_forward_refs_non_strict(annotation, kwargs, expected):
     assert ann == expected
 
 
+@pytest.mark.parametrize(
+    "annotation, expected",
+    [
+        ("List[Foo]", List["Foo"]),  # type: ignore[Foo-not-defined]
+    ],
+)
+def test_partially_resolve_forward_refs(annotation: str, expected):
+    ann = resolve_forward_refs(annotation, mode="ast", strict=False)
+    assert ann == expected
+
+
 def no_args():
     pass
 
